@@ -7,6 +7,7 @@ use std::io;
 use std::io::Read;
 use thiserror::Error;
 
+pub mod file;
 mod lines;
 pub mod print;
 mod sink;
@@ -14,15 +15,27 @@ mod sink;
 #[cfg(test)]
 mod testutil;
 
-/// Error represents the possible errors that can occur during the search process. See `scan_pattern` for more details.
+/// `Error` represents the possible errors that can occur during the search process.
 #[derive(Error, Debug)]
 pub enum Error {
+    /// Parsing the given regular expression failed.
     #[error("Regular expression engine failed: {0}")]
-    RegexError(regex::Error),
+    RegexError(
+        /// The original i/o error that caused the print failure.
+        regex::Error,
+    ),
+    /// The search process encountered a fatal error. This is likely an i/o error, but it is not necessarily.
     #[error("Search process failed: {0}")]
-    SearchError(String),
+    SearchError(
+        /// An error message provided by the underlying grep library.
+        String,
+    ),
+    /// Printing to the given printer failed due to an i/o error. The original error is wrapped in the variant
     #[error("Printing results failed: {0}")]
-    PrintFailure(io::Error),
+    PrintFailure(
+        /// The original i/o error that caused the print failure.
+        io::Error,
+    ),
 }
 
 impl From<sink::Error> for Error {
@@ -42,11 +55,11 @@ impl From<regex::Error> for Error {
 
 /// `scan_pattern` will print a reader's contents, while also scanning its contents for a regular expression.
 /// Lines that match this pattern will be highlighted in the output.
-/// A convenience wrapper for `scan_pattern_to_printer` that will print to stdout.
+/// A convenience wrapper for [`scan_pattern_to_printer`] that will print to stdout.
 ///
 /// # Errors
 ///
-/// See `scan_pattern_to_printer`
+/// See [`scan_pattern_to_printer`]
 pub fn scan_pattern<R: Read>(reader: R, pattern: &str) -> Result<(), Error> {
     scan_pattern_to_printer(reader, pattern, StdoutPrinter::new())
 }
@@ -59,10 +72,14 @@ pub fn scan_pattern<R: Read>(reader: R, pattern: &str) -> Result<(), Error> {
 ///
 /// # Errors
 ///
-/// There are three general error cases
+/// There are four general error cases
 /// - An invalid regular expression
-/// - I/O errors in scanning from the `Read`
 /// - An error produced by the underlying grep library during the search
+/// - I/O errors in scanning from the [`Read`] (these will be manifested as search errors, as the search process is what
+///   performs the reading).
+/// - A failure to print to the given printer
+///
+/// See [enum@Error] for more details.
 pub fn scan_pattern_to_printer<R: Read, P: Printer>(
     reader: R,
     pattern: &str,
@@ -81,7 +98,7 @@ mod tests {
     use super::*;
     use crate::testutil;
     use std::io;
-    use stringreader::StringReader;
+    use std::io::Cursor;
     use test_case::test_case;
     use testutil::mock_print::MockPrinter;
 
@@ -102,7 +119,7 @@ mod tests {
     #[test]
     fn test_highlights_matches() {
         let mock_printer = MockPrinter::default();
-        let mut lipsum_reader = StringReader::new(SEARCH_TEXT);
+        let mut lipsum_reader = Cursor::new(SEARCH_TEXT);
         let res = scan_pattern_to_printer(
             &mut lipsum_reader,
             r#""?computable"?\snumbers"#,
@@ -141,7 +158,7 @@ mod tests {
     #[test]
     fn case_insensitive_pattern_matches() {
         let mock_printer = MockPrinter::default();
-        let mut lipsum_reader = StringReader::new(SEARCH_TEXT);
+        let mut lipsum_reader = Cursor::new(SEARCH_TEXT);
         // This test is a little bit of a cheat, because it doesn't test what's actually inputted by the CLI,
         // but it does make sure the functionality works as expected
         let res = scan_pattern_to_printer(&mut lipsum_reader, "(?i)INTEGRAL", &mock_printer);
@@ -186,7 +203,7 @@ mod tests {
         let broken_pipe_err =
             print::Error::from(io::Error::new(io::ErrorKind::BrokenPipe, "broken pipe"));
         mock_printer.fail_next(broken_pipe_err);
-        let mut lipsum_reader = StringReader::new(SEARCH_TEXT);
+        let mut lipsum_reader = Cursor::new(SEARCH_TEXT);
         let res = scan_pattern_to_printer(&mut lipsum_reader, pattern, &mock_printer);
 
         assert!(!res.is_err(), "failed to search: {:?}", res.unwrap_err());
